##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
""" MySAP SSO Cookie Crumbler: patch the original Cookie Crumber with SSO support.

$Id: MySapSsoCookieCrumbler.py,v 1.1 2005/07/04 11:02:37 admin Exp $
"""

from base64 import encodestring, decodestring
from urllib import quote, unquote
import md5

from DateTime import DateTime
from ZPublisher.HTTPRequest import HTTPRequest

from urllib2 import urlopen, Request

from TicketInfo import TicketInfo

# import all from CookieCrumbler
from Products.CookieCrumbler.CookieCrumbler import *

# new constant
ATTEMPT_TICKET = 3     # Attempt to authenticate via sso-ticket

# extend properties of CookieCrumbler
CookieCrumbler._properties += \
		  ({'id':'mysap_cookie', 'type': 'string', 'mode':'w',
                    'label':'Authentication ticket cookie name'},
                   {'id':'ticket_verifier_url', 'type': 'string', 'mode':'w',
                    'label':'Ticket Verifier URL'},
                  )

CookieCrumbler.mysap_cookie = 'MYSAPSSO2'
CookieCrumbler.ticket_verifier_url = 'http://localhost:4080/verify'

def _verifyTicket( self, ticket ):
        """ Verify Ticket with external verification service """

        if ticket is None or ticket == '':
            return None

        req = Request( self.ticket_verifier_url, data=ticket )
        req.add_header( 'Cookie', '%s=%s;' % (self.mysap_cookie, ticket) )

        try:
            resp = urlopen(req)
            result = resp.read()
            print result[:64]
            return TicketInfo(result)
        except Exception, ex:
            print "%s: %s" % (self.mysap_cookie, ex)
            return None


def _modifyRequest(self, req, resp):
        """Copies cookie-supplied credentials to the basic auth fields.

        Returns a flag indicating what the user is trying to do with
        cookies: ATTEMPT_NONE, ATTEMPT_LOGIN, or ATTEMPT_RESUME.  If
        cookie login is disabled for this request, raises
        CookieCrumblerDisabled.
        """
        if (req.__class__ is not HTTPRequest
            or not req['REQUEST_METHOD'] in ('HEAD', 'GET', 'PUT', 'POST')
            or req.environ.has_key('WEBDAV_SOURCE_PORT')):
            raise CookieCrumblerDisabled

        # attempt may contain information about an earlier attempt to
        # authenticate using a higher-up cookie crumbler within the
        # same request.
        attempt = getattr(req, '_cookie_auth', ATTEMPT_NONE)

        if attempt == ATTEMPT_NONE:
            if req._auth:
                # An auth header was provided and no cookie crumbler
                # created it.  The user must be using basic auth.
                raise CookieCrumblerDisabled

	    ticket_info = None

	    if req.has_key(self.mysap_cookie) and self.ticket_verifier_url:
		cookie_val = req.get(self.mysap_cookie)
		m = md5.new()
		m.update(cookie_val)
		cookie_hash = m.hexdigest()

		#print cookie_val

		if req.SESSION.has_key(cookie_hash):
		    #print "get ticket_info %s" % cookie_hash
		    ticket_info = req.SESSION.get(cookie_hash)
		else:
		    ticket_info = self._verifyTicket(cookie_val)

		    if ticket_info:
			#print "set ticket_info %s" % cookie_hash
		     	req.SESSION.set(cookie_hash, ticket_info)

	    if ticket_info:
                    self.delRequestVar(req, self.mysap_cookie)
		    name = ticket_info.getPortalUser()
                    ac = encodestring('%s:__ssoticket__=%s' % (name,cookie_hash)).rstrip()
                    req._auth = 'Basic %s' % ac
                    resp._auth = 1
                    #print req._auth
                    #print name
		    attempt = ATTEMPT_TICKET

            elif req.has_key(self.pw_cookie) and req.has_key(self.name_cookie):
                # Attempt to log in and set cookies.
                attempt = ATTEMPT_LOGIN
                name = req[self.name_cookie]
                pw = req[self.pw_cookie]
		if pw[:13] == '__ssoticket__': pw = None
                ac = encodestring('%s:%s' % (name, pw)).rstrip()
                req._auth = 'Basic %s' % ac
                resp._auth = 1
                if req.get(self.persist_cookie, 0):
                    # Persist the user name (but not the pw or session)
                    expires = (DateTime() + 365).toZone('GMT').rfc822()
                    resp.setCookie(self.name_cookie, name,
                                   path=self.getCookiePath(),
                                   expires=expires)
                else:
                    # Expire the user name
                    resp.expireCookie(self.name_cookie,
                                      path=self.getCookiePath())
                method = self.getCookieMethod( 'setAuthCookie'
                                             , self.defaultSetAuthCookie )
                method( resp, self.auth_cookie, quote( ac ) )
                self.delRequestVar(req, self.name_cookie)
                self.delRequestVar(req, self.pw_cookie)

            elif req.has_key(self.auth_cookie):
                # Attempt to resume a session if the cookie is valid.
                # Copy __ac to the auth header.
                ac = unquote(req[self.auth_cookie])
                if ac and ac != 'deleted':
                    try:
                        decodestring(ac)
                    except:
                        # Not a valid auth header.
                        pass
                    else:
                        attempt = ATTEMPT_RESUME
                        req._auth = 'Basic %s' % ac
                        resp._auth = 1
                        self.delRequestVar(req, self.auth_cookie)
                        method = self.getCookieMethod(
                            'twiddleAuthCookie', None)
                        if method is not None:
                            method(resp, self.auth_cookie, quote(ac))

        req._cookie_auth = attempt
        return attempt


def _call(self, container, req):
        '''The __before_publishing_traverse__ hook.'''
        resp = self.REQUEST['RESPONSE']
        try:
            attempt = self.modifyRequest(req, resp)
        except CookieCrumblerDisabled:
            return
        if req.get('disable_cookie_login__', 0):
            return

        if (self.unauth_page or
            attempt in [ATTEMPT_LOGIN, ATTEMPT_NONE, ATTEMPT_TICKET]):
            # Modify the "unauthorized" response.
            req._hold(ResponseCleanup(resp))
            resp.unauthorized = self.unauthorized
            resp._unauthorized = self._unauthorized

        if attempt != ATTEMPT_NONE and attempt != ATTEMPT_TICKET:
            # Trying to log in or resume a session
            try:
              if self.cache_header_value:
                # we don't want caches to cache the resulting page
                resp.setHeader('Cache-Control', self.cache_header_value)
                # demystify this in the response.
                resp.setHeader('X-Cache-Control-Hdr-Modified-By',
                               'CookieCrumbler')
            except:
              pass

            phys_path = self.getPhysicalPath()
            if self.logout_page:
                # Cookies are in use.
                page = getattr(container, self.logout_page, None)
                if page is not None:
                    # Provide a logout page.
                    req._logout_path = phys_path + ('logout',)
            req._credentials_changed_path = (
                phys_path + ('credentialsChanged',))

CookieCrumbler._verifyTicket = _verifyTicket
CookieCrumbler.modifyRequest = _modifyRequest
CookieCrumbler.__call__      = _call
